#include "regs.h"
#include "emac.h"
#include "udpip.h"
#include "ssc.h"
#define NULL 0

void Clock_Init()
{
	/* let the watchdog sleep */
	WDT_MR = WDT_MR_WDDIS;

	/* Clock configuration: Use PLL, DIV=3, MUL=16; MCLK Prescaler=2       */
	/* Result: F_PLL = 16 / 3 * 18.432 MHz = 98.304 MHz, MCLK = 49.152 MHz */
	/* Sample rate of SSC will be MCLK / 2 = 24.576 MHz */

	PMC_MCKR &= ~0x03;                 /* CSS: use slow clock */
	while( !(PMC_SR & PMC_SR_MCKRDY) ) /* wait for clock ready */
		;

	CKGR_MOR = CKGR_MOR_MOSCEN | (64 * CKGR_MOR_OSCOUNT); /* start main osc */
	while( !(PMC_SR & PMC_SR_MOSCS) )  /* wait until startup time is over */
		;

	CKGR_PLLR = 0x0f3f03;              /* configure PLL: 16/3 * 18.432 MHz */
	while( !(PMC_SR & PMC_SR_LOCK) )   /* wait for PLL lock */
		;

	PMC_MCKR = 0x04;                   /* PRES: set prescaler: div by 2 */
	while( !(PMC_SR & PMC_SR_MCKRDY) ) /* wait for main clock to become ready */
		;
	PMC_MCKR = 0x07;                   /* CSS: switch to PLL */	
	while( !(PMC_SR & PMC_SR_MCKRDY) ) /* wait for main clock to become ready */
		;

	PMC_PCER = (1 << PERIPH_ID_PIOA) | (1 << PERIPH_ID_PIOB); /* PIO clocks on */
}

int main()
{
#define N_SHOVEL 256
	unsigned int shovel_buffer[2][N_SHOVEL];
	unsigned int idx;

	Clock_Init();
	EMAC_Init();
	UDPIP_Init();
	SSC_Init();
	PDC_RPR(SSC)  = (unsigned int)&shovel_buffer[0][0];
	PDC_RNPR(SSC) = (unsigned int)&shovel_buffer[1][0];
	PDC_RCR(SSC)  = N_SHOVEL;
	PDC_RNCR(SSC) = N_SHOVEL;
	PDC_PTCR(SSC) = PDC_PTCR_RXTEN;
	idx = 0;
	for(;;)
	{
		while( PDC_RNCR(SSC) != 0 )
			;
		UDPIP_Send((unsigned char *)&shovel_buffer[idx][0], 4 * N_SHOVEL);
		PDC_RNPR(SSC) = (unsigned int)&shovel_buffer[idx][0];
		PDC_RNCR(SSC) = N_SHOVEL;
		idx = (idx + 1) % 2;
	}
	return 0;
}
